local logger = require('saotrace.log').init( "logfile", 'DEBUG' )

local aperture = require( 'saotrace.aperture.mock' )
local ideal = require('saotrace.aperture.ideal').create_baffles

local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local db = table.concat( { srcdir, 'tests', 'data', 'geometry.rdb'}, '/' )

local giter = require('saotrace.geometry').read{ file = db, mode = 'iterator' }

local optic = {}

for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

function _setup () 
      aperture._reset_stack()
end

describe( "outermost", function ()

   setup( _setup )


   it ( "test for correct radii of generated circles", function ()

   ideal{ shell = 1,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   	local stack = aperture._get_stack()
	assert.are.equal( optic.p[3].rho_f, stack[4][2] )
	assert.are.equal( optic.p[1].rho_f, stack[8][2] )
   end)

end)

describe( "middle", function ()

   setup( _setup )


   it ( "test for correct radii of generated circles", function ()

   ideal{ shell = 3,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }


   	local stack = aperture._get_stack()
        assert.are.equal( optic.p[4].rho_f, stack[4][2] )
        assert.are.equal( optic.p[3].rho_f, stack[8][2] )
   end)

end)

describe( "inner_disk", function( )

   setup( _setup )


   it ( "test for correct radii of generated circles", function ()

   ideal{ shell = 6,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db,
	  inner_disk = { z = 20,
			 r = 10 },
       }


	local stack = aperture._get_stack()
	assert.are.equal( 20, stack[3][4] )
	assert.are.equal( 10, stack[4][2] )
        assert.are.equal( optic.p[6].rho_f, stack[8][2] )
   end)


end)

describe( "no_inner_disk", function ()

   setup( _setup )


   it ( "test for correct radii of generated circles", function ()

   ideal{ shell = 6,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }


	local stack = aperture._get_stack()
	assert.are.equal( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f), stack[4][2] )
	assert.are.equal( optic.p[6].rho_f, stack[8][2]  )
   end)

end)
